@@ -1,7 +1,7 @@
 #' Select top (or bottom) n rows (by value)
 #'
 #' @description
-#' \Sexpr[results=rd, stage=render]{lifecycle::badge("superseded")}
+#' `r lifecycle::badge("superseded")`
 #' `top_n()` has been superseded in favour of [slice_min()]/[slice_max()].
 #' While it will not be deprecated in the near future, retirement means
 #' that we will only perform critical bug fixes, so we recommend moving to the
@@ -28,18 +28,18 @@
 #' @examples
 #' df <- data.frame(x = c(6, 4, 1, 10, 3, 1, 1))
 #'
-#' df %>% top_n(2)  # highest values
-#' df %>% top_n(-2) # lowest values
+#' df |> top_n(2)  # highest values
+#' df |> top_n(-2) # lowest values
 #' # now use
-#' df %>% slice_max(x, n = 2)
-#' df %>% slice_min(x, n = 2)
+#' df |> slice_max(x, n = 2)
+#' df |> slice_min(x, n = 2)
 #'
 #' # top_frac() -> prop argument of slice_min()/slice_max()
-#' df %>% top_frac(.5)
+#' df |> top_frac(.5)
 #' # ->
-#' df %>% slice_max(x, prop = 0.5)
+#' df |> slice_max(x, prop = 0.5)
 top_n <- function(x, n, wt) {
-  lifecycle::signal_superseded("1.0.0", "top_n()", "slice_max()")
+  lifecycle::signal_stage("superseded", "top_n()")
   wt <- enquo(wt)
   if (quo_is_missing(wt)) {
     vars <- tbl_vars(x)
