@@ -1,4 +1,3 @@
-
 commas <- function(...) paste0(..., collapse = ", ")
 
 compact_null <- function(x) {
@@ -9,12 +8,19 @@ paste_line <- function(...) {
   paste(chr(...), collapse = "\n")
 }
 
+vec_paste0 <- function(...) {
+  args <- vec_recycle_common(...)
+  exec(paste0, !!!args)
+}
+
 # Until vctrs::new_data_frame() forwards row names automatically
-dplyr_new_data_frame <- function(x = data.frame(),
-                                 n = NULL,
-                                 ...,
-                                 row.names = NULL,
-                                 class = NULL) {
+dplyr_new_data_frame <- function(
+  x = data.frame(),
+  n = NULL,
+  ...,
+  row.names = NULL,
+  class = NULL
+) {
   row.names <- row.names %||% .row_names_info(x, type = 0L)
 
   new_data_frame(
@@ -99,7 +105,8 @@ maybe_restart <- function(restart) {
 
 expr_substitute <- function(expr, old, new) {
   expr <- duplicate(expr)
-  switch(typeof(expr),
+  switch(
+    typeof(expr),
     language = node_walk_replace(node_cdr(expr), old, new),
     symbol = if (identical(expr, old)) return(new)
   )
@@ -107,16 +114,42 @@ expr_substitute <- function(expr, old, new) {
 }
 node_walk_replace <- function(node, old, new) {
   while (!is_null(node)) {
-    switch(typeof(node_car(node)),
-      language = if (!is_call(node_car(node), c("~", "function")) || is_call(node_car(node), "~", n = 2)) node_walk_replace(node_cdar(node), old, new),
-      symbol = if (identical(node_car(node), old)) node_poke_car(node, new)
+    switch(
+      typeof(node_car(node)),
+      language = if (
+        !is_call(node_car(node), c("~", "function")) ||
+          is_call(node_car(node), "~", n = 2)
+      ) {
+        node_walk_replace(node_cdar(node), old, new)
+      },
+      symbol = if (identical(node_car(node), old)) {
+        node_poke_car(node, new)
+      }
     )
     node <- node_cdr(node)
   }
 }
 
-cli_collapse <- function(x, last = " and ") {
-  cli::cli_vec(x, style = list("vec-last" = last))
+cli_collapse <- function(x, last = " and ", sep2 = " and ") {
+  cli::cli_vec(x, style = list("vec-last" = last, "vec-sep2" = sep2))
+}
+
+# A version of `cli::format_inline()` that formats each string individually.
+# Useful for constructing error bullets ahead of time.
+cli_format_each_inline <- function(
+  ...,
+  .envir = parent.frame(),
+  collapse = TRUE,
+  keep_whitespace = TRUE
+) {
+  map_chr(c(...), function(x) {
+    cli::format_inline(
+      x,
+      .envir = .envir,
+      collapse = collapse,
+      keep_whitespace = keep_whitespace
+    )
+  })
 }
 
 with_no_rlang_infix_labeling <- function(expr) {
