@@ -1,13 +1,6 @@
 .onLoad <- function(libname, pkgname) {
   ns_dplyr <- ns_env(pkgname)
 
-  op <- options()
-  op.dplyr <- list(
-    dplyr.show_progress = TRUE
-  )
-  toset <- !(names(op.dplyr) %in% names(op))
-  if (any(toset)) options(op.dplyr[toset])
-
   .Call(dplyr_init_library, ns_dplyr, ns_env("vctrs"), ns_env("rlang"))
 
   # TODO: For `arrange()`, `group_by()`, `with_order()`, and `nth()` until vctrs
@@ -40,13 +33,16 @@
   setHook(packageEvent("plyr", "attach"), NULL, "replace")
 }
 
-import_vctrs <- function(name) {
-  import_from(name, "vctrs")
+import_vctrs <- function(name, optional = FALSE) {
+  import_from(name, "vctrs", optional = optional)
 }
-import_from <- function(name, package) {
+import_from <- function(name, package, optional = FALSE) {
   ns <- getNamespace(package)
 
   if (!exists(name, mode = "function", envir = ns, inherits = FALSE)) {
+    if (optional) {
+      return(NULL)
+    }
     abort(sprintf("No such '%s' function: `%s()`.", package, name))
   }
 
