@@ -18,13 +18,17 @@ get_lang_strings <- function() {
 
 get_native_lang_string <- function() {
   lang_strings <- get_lang_strings()
-  if (length(lang_strings$same) == 0) testthat::skip("No native language string available")
+  if (length(lang_strings$same) == 0) {
+    testthat::skip("No native language string available")
+  }
   lang_strings$same[[1L]]
 }
 
 get_alien_lang_string <- function() {
   lang_strings <- get_lang_strings()
-  if (length(lang_strings$different) == 0) testthat::skip("No alien language string available")
+  if (length(lang_strings$different) == 0) {
+    testthat::skip("No alien language string available")
+  }
   lang_strings$different[[1L]]
 }
 
@@ -52,12 +56,13 @@ non_utf8_encoding <- function(enc = NULL) {
   if (!l10n_info()$`UTF-8`) {
     return(Sys.getlocale("LC_CTYPE"))
   }
-  enc <- enc %||% c(
-    "en_US.ISO8859-1",
-    "en_US.ISO8859-15",
-    "fr_CH.ISO8859-1",
-    "fr_CH.ISO8859-15"
-  )
+  enc <- enc %||%
+    c(
+      "en_US.ISO8859-1",
+      "en_US.ISO8859-15",
+      "fr_CH.ISO8859-1",
+      "fr_CH.ISO8859-15"
+    )
   available <- vapply(enc, has_ctype_locale, logical(1))
   if (any(available)) {
     enc[available][1]
